<?php

namespace EventPortal\Domain\Event\Models;

use EventPortal\Concerns\HasStatus;
use EventPortal\Domain\Component\Option;
use EventPortal\Domain\Venue\Models\Venue;
use Iben\Statable\Statable;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\Pivot;

class EventVenue extends Pivot
{
    use Statable, HasStatus;

    public const KEY = 'eventVenue';

    public $incrementing = true;

    protected $with = [
        'event',
        'venue',
        'venue.address',
    ];

    public function event(): BelongsTo
    {
        return $this->belongsTo(Event::class);
    }

    public function venue(): BelongsTo
    {
        return $this->belongsTo(Venue::class)->with('address');
    }

    public function asOption(): Option
    {
        return Option::make($this->venue->name)->value($this->id);
    }

    public function scopeForEvent(Builder $query, Event $event): Builder
    {
        return $query->where('event_id', $event->id);
    }

    public function scopeForEventAndVenue(Builder $query, Event $event, Venue $venue): Builder
    {
        return $query
            ->where('event_id', $event->id)
            ->where('venue_id', $venue->id);
    }

    public static function allAsOptions(Event $event): array
    {
        return array_values(
            self::forEvent($event)
                ->get()
                ->sortBy(static function (EventVenue $eventVenue) {
                    return $eventVenue->venue->name;
                })
                ->map(static function (EventVenue $eventVenue) {
                    return $eventVenue->asOption();
                })
                ->toArray()
        );
    }
}
