<?php

namespace EventPortal\Tests\Support\Factories;

use EventPortal\Domain\Agenda\Models\Agenda;
use EventPortal\Domain\Agenda\Models\Pathway;
use EventPortal\Domain\Agenda\Models\Session;
use EventPortal\Domain\Speaker\Models\Speaker;
use EventPortal\Domain\User\Models\User;
use EventPortal\Domain\Venue\Models\Venue;

class SessionFactory
{
    /** @var \EventPortal\Domain\Agenda\Models\Session|null */
    public $session;

    /** @var \EventPortal\Domain\Agenda\Models\Agenda */
    public $agenda;

    /** @var \EventPortal\Domain\Agenda\Models\Pathway */
    public $pathway;

    public function __construct(Agenda $agenda, Pathway $pathway)
    {
        $this->agenda = $agenda;
        $this->pathway = $pathway;
    }

    public function create(array $data = []): self
    {
        $data = array_merge([
            'pathway_id' => $this->pathway->id,
            'agenda_id' => $this->agenda->id,
        ], $data);

        $this->session = factory(Session::class)->create($data);

        return $this;
    }

    public function createSpeakerAndUser(array $speakerData = [], array $userData = []): self
    {
        $data = array_merge(
            $speakerData,
            [
                'user_id' => factory(User::class)->create($userData)->id,
            ],
        );

        $speaker = factory(Speaker::class)->create($data);

        $this->session->eventSpeakers()->create([
            'speaker_id' => $speaker->id,
            'event_id' => $this->session->agenda->event->id,
        ]);

        return $this;
    }

    public function createVenue(array $data = []): self
    {
        $venue = factory(Venue::class)->create($data);

        $eventVenue = $this->session->eventVenue()->create([
            'venue_id' => $venue->id,
            'event_id' => $this->session->agenda->event->id,
        ]);

        $this->session->eventVenue()->associate($eventVenue);
        $this->session->save();

        return $this;
    }
}
